#include <errno.h>
#include <pwd.h>
#include <grp.h>
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>           /* uintmax_t */

#include "shim.h"

static void
fish_open_archive_pipeopen (struct vfs_s_super *super)
{
    char gbuf[10];
    const char *argv[10];       /* All of 10 is used now */
    const char *xsh = (super->path_element->port == FISH_FLAG_RSH ? "rsh" : "ssh");
    int i = 0;

    argv[i++] = xsh;
    if (super->path_element->port == FISH_FLAG_COMPRESSED)
        argv[i++] = "-C";

    if (super->path_element->port > FISH_FLAG_RSH)
    {
        argv[i++] = "-p";
        g_snprintf (gbuf, sizeof (gbuf), "%d", super->path_element->port);
        argv[i++] = gbuf;
    }

    /*
     * Add the user name to the ssh command line only if it was explicitly
     * set in vfs URL. rsh/ssh will get current user by default
     * plus we can set convenient overrides in  ~/.ssh/config (explicit -l
     * option breaks it for some)
     */

    if (super->path_element->user != NULL)
    {
        argv[i++] = "-l";
        argv[i++] = super->path_element->user;
    }
    else
    {
        /* The rest of the code assumes it to be a valid username */
        super->path_element->user = vfs_get_local_username ();
    }

    argv[i++] = super->path_element->host;
    argv[i++] = "echo FISH:; /bin/sh";
    argv[i++] = NULL;

    fish_pipeopen (super, xsh, argv);
}


static void
//fish_pipeopen (struct vfs_s_super *super, const char *path, const char *argv[])
fish_pipeopen (const char *path, const char *argv[])
{
    int fileset1[2], fileset2[2];
    int res;

// ***
    if ((pipe (fileset1) < 0) || (pipe (fileset2) < 0))
//        vfs_die ("Cannot pipe(): %m.");
exit(-10);

    res = fork ();

    if (res != 0)
    {
// ***
        if (res < 0)
//            vfs_die ("Cannot fork(): %m.");
exit(-20);
        /* We are the parent */
        close (fileset1[0]);
        //FISH_SUPER (super)->sockw = fileset1[1];
        close (fileset2[1]);
        //FISH_SUPER (super)->sockr = fileset2[0];
    }
    else
    {
        res = dup2 (fileset1[0], STDIN_FILENO);
        close (fileset1[0]);
        close (fileset1[1]);
        res = dup2 (fileset2[1], STDOUT_FILENO);
        close (STDERR_FILENO);
        /* stderr to /dev/null */
        res = open ("/dev/null", O_WRONLY);
        close (fileset2[0]);
        close (fileset2[1]);
        execvp (path, (char **) argv);
// ***
//        my_exit (3);
		exit(-30);
    }
}

void main() {
}
